<?php
namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Voucher;
use App\Models\Accounting;

class CashbookExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $startDate;
    protected $endDate;
    protected $accountId;
    protected $openBal;
    protected $closeBal;
        
    // Accept start and end dates in the constructor
    public function __construct($startDate, $endDate, $accountId)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
        $this->openBal = 0;
        $this->closeBal = 0;
    }
        
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $open_bal = $this->getOpeningBalance();
        $this->openBal = $open_bal;
            
        $vouchers = Voucher::whereNull('deleted_at')
                    ->where('vdate', '>=', $this->startDate)
                    ->where('vdate', '<=', $this->endDate)
                    ->where('account_id', $this->accountId)
                    ->where(function ($query) {
                        $query->where('cr_ledger', 1)
                            ->orWhere('dr_ledger', 1);
                    })
                    ->get();
                    
        return $vouchers->map(function ($voucher) use (&$open_bal) {
            $dr = $cr = 0;
            $ledger_name = '';
            if ($voucher->dr_ledger == 1) {
                $dr = $voucher->vamount;
                $ledger_name = 'To ' . $voucher->credit->ledger_name;
            } else if ($voucher->cr_ledger == 1) {
                $cr = $voucher->vamount;
                $ledger_name = 'By ' . $voucher->debit->ledger_name;
            }
            
            $open_bal += ($dr - $cr);
            $this->closeBal = $open_bal;
                
            return [
                $voucher->vdate_formatted,
                $ledger_name,
                $voucher->vno,
                $voucher->vtype->vtype_name,
                number_format($dr, 2),
                number_format($cr, 2),
                number_format($open_bal, 2)
            ];
        });
    }
        
    public function headings(): array
    {
        return [
            'Date',
            'Particulars',
            'Voucher No',
            'Voucher Type',
            'Debit',
            'Credit',
            'Closing Balance'
        ];
    }
        
    public function title(): string
    {
        return 'Cash Book';
    }
        
    public function styles(Worksheet $sheet)
    {
        // Set column widths
        $sheet->getColumnDimension('A')->setWidth(15);
        $sheet->getColumnDimension('B')->setWidth(35);
        $sheet->getColumnDimension('C')->setWidth(15);
        $sheet->getColumnDimension('D')->setWidth(20);
        $sheet->getColumnDimension('E')->setWidth(15);
        $sheet->getColumnDimension('F')->setWidth(15);
        $sheet->getColumnDimension('G')->setWidth(15);
            
        // Style for headings (row 1)
        $sheet->getStyle('A1:G1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00', // Yellow background for headings
                ]
            ]
        ]);
            
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;
                
                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Cashbook From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:G1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $sheet->insertNewRowBefore(3, 1);
                $sheet->setCellValue('B3', 'Opening Balance');
                $sheet->setCellValue('E3', $this->openBal);
                $sheet->setCellValue('G3', $this->openBal);
                    
                $lastRow = $sheet->getHighestRow() + 1; // Get the last row after data
                $sheet->setCellValue('B'.$lastRow, 'Total');
                $sheet->setCellValue('G'.$lastRow, $this->closeBal);
                
                $sheet->getStyle('A2:G'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);
                
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
                
                $sheet->getStyle('A3:D'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'FFFFFF', // Yellow background for headings
                        ]
                    ]
                ]);
                
                $sheet->getStyle('E3:G'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'FFFFFF', // Yellow background for headings
                        ]
                    ]
                ]);
                
                $sheet->getStyle('A'.$lastRow.':G'.$lastRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ]
                ]);
            }
        ];
    }
        
    /**
     * Calculate the opening balance for the sheet.
     * @return float
     */
    public function getOpeningBalance()
    {
        $pre_date = date('Y-m-d', (strtotime($this->startDate) - 86400));
            
        $pre_open_bal = Accounting::openingBalance($pre_date, $this->accountId, 0, 1);
        $cur_open_bal = Accounting::openingBalance($this->endDate, $this->accountId, 0, 1, $this->startDate);
        $ledger_open_bal = $pre_open_bal + $cur_open_bal;
            
        $voucher_open_bal = Accounting::voucherBalance(1, $pre_date, $this->accountId, 0);
            
        return $ledger_open_bal + ($voucher_open_bal['dr_amount'] - $voucher_open_bal['cr_amount']);
    }
}
