<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Voucher;

class DaybookExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $startDate;
    protected $endDate;
    protected $accountId;

    // Accept start and end dates in the constructor
    public function __construct($startDate, $endDate, $accountId)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        // Get the data from the Voucher model with the necessary filters
        $vouchers = Voucher::join('vouchertypes', 'vouchers.vtype_id', '=', 'vouchertypes.id')
            ->where('vamount', '>', 0)
            ->where('account_id', $this->accountId)
            ->whereBetween('vdate', [$this->startDate, $this->endDate])
            ->select('vouchers.*', 'vouchertypes.vid')
            ->get(); // Get all the results at once
        
        // Map over the vouchers and return the data in the desired format
        return $vouchers->map(function ($voucher, $index) {
            $particulars = $voucher->credit->ledger_name." To ".$voucher->debit->ledger_name;
            $dr_amount = $cr_amount = 0; 
            if($voucher->vid == 2 || $voucher->vid == 3){
                $cr_amount = $voucher->vamount;
            }else{
                $dr_amount = $voucher->vamount;
            }
            
            return [
                $index + 1,
                $voucher->vno,
                $voucher->vdate_formatted,
                $voucher->vtype->vtype_name,
                $particulars,
                number_format($dr_amount,2),
                number_format($cr_amount,2),
            ];
        });
    }
    
    public function headings(): array
    {
        return [
            '#',
            'Voucher No',
            'Date',
            'Voucher Type',
            'Particulars',
            'Debit',
            'Credit',
        ];
    }
    
    public function title(): string
    {
        return 'Daybook Report';
    }
    
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(10); // Serial Number column
        $sheet->getColumnDimension('B')->setWidth(15); // Voucher No column
        $sheet->getColumnDimension('C')->setWidth(15); // Date column
        $sheet->getColumnDimension('D')->setWidth(20); // Voucher Type column
        $sheet->getColumnDimension('E')->setWidth(40); // Particulars column
        $sheet->getColumnDimension('F')->setWidth(15); // Debit column
        $sheet->getColumnDimension('G')->setWidth(15); // Credit column
        
        $sheet->getStyle('A1:G1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00', // Yellow background for headings
                ]
            ]
        ]);
        
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;
                    
                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Daybook From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:G1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $lastRow = $sheet->getHighestRow();
                $sheet->getStyle('A2:G'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);
                    
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
                
                $sheet->getStyle('A3:D'.$lastRow)->applyFromArray([
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $sheet->getStyle('F3:G'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
            }
        ];
    }
}
